@@ -1,8 +1,8 @@
 /*
  * gbsplay is a Gameboy sound player
  *
- * 2003-2006 (C) by Tobias Diedrich <ranma+gbsplay@tdiedrich.de>
- *                  Christian Garbs <mitch@cgarbs.de>
+ * 2003-2006,2013 (C) by Tobias Diedrich <ranma+gbsplay@tdiedrich.de>
+ *                       Christian Garbs <mitch@cgarbs.de>
  * Licensed under GNU GPL.
  */
 
@@ -24,19 +24,22 @@
 #define GBS_MAGIC		"GBS"
 #define GBS_EXTHDR_MAGIC	"GBSX"
 
+/* called with subsong in a, load address in hl */
 static const uint8_t playercode[] = {
+/* Part 1, hardware init */
 	0xf5,              /* 0050:  push af         */
 	0xe5,              /* 0051:  push hl         */
 	0x01, 0x30, 0x00,  /* 0052:  ld   bc, 0x0030 */
 	0x11, 0x10, 0xff,  /* 0055:  ld   de, 0xff10 */
 	0x21, 0x9f, 0x00,  /* 0058:  ld   hl, 0x009f */
+	                   /*   l1:                  */
 	0x2a,              /* 005b:  ldi  a, [hl]    */
 	0x12,              /* 005c:  ld   [de], a    */
 	0x13,              /* 005d:  inc  de         */
 	0x0b,              /* 005e:  dec  bc         */
 	0x78,              /* 005f:  ld   a, b       */
 	0xb1,              /* 0060:  or   a, c       */
-	0x20, 0xf8,        /* 0061:  jr nz $-0x08    */
+	0x20, 0xf8,        /* 0061:  jr nz l1 ; ($-8)*/
 	0xe1,              /* 0063:  pop  hl         */
 	0xe5,              /* 0064:  push hl         */
 	0x01, 0x0e, 0x00,  /* 0065:  ld   bc, 0x000e */
@@ -48,11 +51,13 @@ static const uint8_t playercode[] = {
 	0x11, 0xff, 0xff,  /* 006f:  ld   de, 0xffff */
 	0xcb, 0x57,        /* 0072:  bit  2, a       */
 	0x3e, 0x01,        /* 0074:  ld   a, 0x01    */
-	0x28, 0x02,        /* 0076:  jr z $+2        */
+	0x28, 0x02,        /* 0076:  jr z l2 ; ($+2) */
 	0x3e, 0x04,        /* 0078:  ld   a, 0x04    */
+	                   /*   l2:                  */
 	0x12,              /* 007a:  ld   [de], a    */
 	0xe1,              /* 007b:  pop  hl         */
 	0xf1,              /* 007c:  pop  af         */
+/* Part 2, wrapper for replayer code */
 	0x57,              /* 007d:  ld   d, a       */
 	0xe5,              /* 007e:  push hl         */
 	0x01, 0x08, 0x00,  /* 007f:  ld   bc, 0x0008 */
@@ -64,6 +69,7 @@ static const uint8_t playercode[] = {
 	0x01, 0x8c, 0x00,  /* 0087:  ld   bc, 0x008c */
 	0xc5,              /* 008a:  push bc         */
 	0xe9,              /* 008b:  jp   hl         */
+	                   /*   l3:                  */
 	0xfb,              /* 008c:  ei              */
 	0x76,              /* 008d:  halt            */
 	0xe1,              /* 008e:  pop  hl         */
@@ -77,15 +83,22 @@ static const uint8_t playercode[] = {
 	0x01, 0x9d, 0x00,  /* 0098:  ld   bc, 0x009d */
 	0xc5,              /* 009b:  push bc         */
 	0xe9,              /* 009c:  jp   hl         */
-	0x18, 0xee,        /* 009d:  jr $-6          */
+	0x18, 0xed,        /* 009d:  jr l3 ; ($-19)  */
 
-	0x80, 0xbf, 0x00, 0x00, 0xbf, /* 009f: initdata */
+/* 009f: initdata
+ *
+ * Initial state of sound hardware registers
+ * and wave pattern memory
+ */
+/* sound registers */
+	0x80, 0xbf, 0x00, 0x00, 0xbf,
 	0x00, 0x3f, 0x00, 0x00, 0xbf,
 	0x7f, 0xff, 0x9f, 0x00, 0xbf,
 	0x00, 0xff, 0x00, 0x00, 0xbf,
 	0x77, 0xf3, 0xf1, 0x00,
 	0x00, 0x00, 0x00, 0x00,
 	0x00, 0x00, 0x00, 0x00,
+/* wave pattern memory, taken from gbsound.txt v0.99.19 (12/31/2002) */
 	0xac, 0xdd, 0xda, 0x48,
 	0x36, 0x02, 0xcf, 0x16,
 	0x2c, 0x04, 0xe5, 0x2c,
@@ -177,17 +190,18 @@ regparm long gbs_step(struct gbs *gbs, long time_to_work)
 
 regparm void gbs_printinfo(struct gbs *gbs, long verbose)
 {
-	printf(_("GBSVersion:     %ld\n"
-	         "Title:          \"%s\"\n"
-	         "Author:         \"%s\"\n"
-	         "Copyright:      \"%s\"\n"
-	         "Load address:   0x%04x\n"
-	         "Init address:   0x%04x\n"
-	         "Play address:   0x%04x\n"
-	         "Stack pointer:  0x%04x\n"
-	         "File size:      0x%08x\n"
-	         "ROM size:       0x%08lx (%ld banks)\n"
-	         "Subsongs:       %ld\n"),
+	printf(_("GBSVersion:       %u\n"
+	         "Title:            \"%s\"\n"
+	         "Author:           \"%s\"\n"
+	         "Copyright:        \"%s\"\n"
+	         "Load address:     0x%04x\n"
+	         "Init address:     0x%04x\n"
+	         "Play address:     0x%04x\n"
+	         "Stack pointer:    0x%04x\n"
+	         "File size:        0x%08x\n"
+	         "ROM size:         0x%08lx (%ld banks)\n"
+	         "Subsongs:         %u\n"
+	         "Default subsong:  %u\n"),
 	       gbs->version,
 	       gbs->title,
 	       gbs->author,
@@ -199,7 +213,8 @@ regparm void gbs_printinfo(struct gbs *gbs, long verbose)
 	       (unsigned int)gbs->filesize,
 	       gbs->romsize,
 	       gbs->romsize/0x4000,
-	       gbs->songs);
+	       gbs->songs,
+	       gbs->defaultsong);
 	if (gbs->version == 2) {
 		printf(_("CRC32:		0x%08lx/0x%08lx (%s)\n"),
 		       (unsigned long)gbs->crc, (unsigned long)gbs->crcnow,
@@ -263,10 +278,14 @@ regparm long gbs_write(struct gbs *gbs, char *name, long version)
 	char strings[65536];
 	long stringofs = 0;
 	long newlen = gbs->filesize;
+	long namelen = strlen(name);
+	char *tmpname = malloc(namelen + sizeof(".tmp\0"));
 
+	memcpy(tmpname, name, namelen);
+	sprintf(&tmpname[namelen], ".tmp");
 	memset(pad, 0xff, sizeof(pad));
 
-	if ((fd = open(name, O_WRONLY|O_CREAT|O_TRUNC, 0644)) == -1) {
+	if ((fd = open(tmpname, O_WRONLY|O_CREAT|O_TRUNC, 0644)) == -1) {
 		fprintf(stderr, _("Could not open %s: %s\n"), name, strerror(errno));
 		return 0;
 	}
@@ -329,7 +348,15 @@ regparm long gbs_write(struct gbs *gbs, char *name, long version)
 			gbs->buf[3] = 1;
 		}
 	}
-	write(fd, gbs->buf, newlen);
+	if (write(fd, gbs->buf, newlen) == newlen) {
+		int ret = 1;
+		close(fd);
+		if (rename(tmpname, name) == -1) {
+			fprintf(stderr, _("Could not rename %s to %s: %s\n"), tmpname, name, strerror(errno));
+			ret = 0;
+		}
+		return ret;
+	}
 	close(fd);
 
 	return 1;
@@ -355,25 +382,38 @@ regparm struct gbs *gbs_open(char *name)
 	}
 	fstat(fd, &st);
 	gbs->buf = buf = malloc(st.st_size);
-	read(fd, buf, st.st_size);
+	if (read(fd, buf, st.st_size) != st.st_size) {
+		fprintf(stderr, _("Could not read %s: %s\n"), name, strerror(errno));
+		return NULL;
+	}
 	if (strncmp(buf, GBS_MAGIC, 3) != 0) {
 		fprintf(stderr, _("Not a GBS-File: %s\n"), name);
 		return NULL;
 	}
-	gbs->version = buf[3];
+	gbs->version = buf[0x03];
 	if (gbs->version != 1) {
-		fprintf(stderr, _("GBS Version %d unsupported.\n"), buf[3]);
+		fprintf(stderr, _("GBS Version %d unsupported.\n"), gbs->version);
 		return NULL;
 	}
 
 	gbs->songs = buf[0x04];
+	if (gbs->songs < 1) {
+		fprintf(stderr, _("Number of subsongs = %d is unreasonable.\n"), gbs->songs);
+		return NULL;
+	}
+
 	gbs->defaultsong = buf[0x05];
+	if (gbs->defaultsong < 1 || gbs->defaultsong > gbs->songs) {
+		fprintf(stderr, _("Default subsong %d is out of range [1..%d].\n"), gbs->defaultsong, gbs->songs);
+		return NULL;
+	}
+
 	gbs->load  = readint(&buf[0x06], 2);
 	gbs->init  = readint(&buf[0x08], 2);
 	gbs->play  = readint(&buf[0x0a], 2);
 	gbs->stack = readint(&buf[0x0c], 2);
-	gbs->tma = buf[0x0e];
-	gbs->tmc = buf[0x0f];
+	gbs->tma = buf[0x0e]; /* Will be programmed by playercode */
+	gbs->tmc = buf[0x0f]; /* Will be programmed by playercode */
 
 	memcpy(gbs->v1strings, &buf[0x10], 32);
 	memcpy(gbs->v1strings+33, &buf[0x30], 32);
@@ -455,7 +495,7 @@ regparm struct gbs *gbs_open(char *name)
 
 	for (i=0; i<8; i++) {
 		long addr = gbs->load + 8*i; /* jump address */
-		gbs->rom[8*i]   = 0xc3; /* jr imm16 */
+		gbs->rom[8*i]   = 0xc3; /* jp imm16 */
 		gbs->rom[8*i+1] = addr & 0xff;
 		gbs->rom[8*i+2] = addr >> 8;
 	}
