@@ -75,10 +75,11 @@ static long redraw = false;
 
 static const char cfgfile[] = ".gbsplayrc";
 
-static char *sound_name = "oss";
+static char *sound_name = PLUGOUT_DEFAULT;
 static char *sound_description;
 static plugout_open_fn  sound_open;
 static plugout_skip_fn  sound_skip;
+static plugout_pause_fn sound_pause;
 static plugout_io_fn    sound_io;
 static plugout_write_fn sound_write;
 static plugout_close_fn sound_close;
@@ -92,16 +93,16 @@ static struct gbhw_buffer buf = {
 
 /* configuration directives */
 static const struct cfg_option options[] = {
+	{ "endian", &endian, cfg_endian },
+	{ "fadeout", &fadeout, cfg_long },
+	{ "loop", &loopmode, cfg_long },
+	{ "output_plugin", &sound_name, cfg_string },
 	{ "rate", &rate, cfg_long },
 	{ "refresh_delay", &refresh_delay, cfg_long },
-	{ "verbosity", &verbosity, cfg_long },
-	{ "endian", &endian, cfg_endian },
-	{ "subsong_timeout", &subsong_timeout, cfg_long },
-	{ "subsong_gap", &subsong_gap, cfg_long },
-	{ "fadeout", &fadeout, cfg_long },
 	{ "silence_timeout", &silence_timeout, cfg_long },
-	{ "output_plugin", &sound_name, cfg_string },
-	{ "loop", &loopmode, cfg_long },
+	{ "subsong_gap", &subsong_gap, cfg_long },
+	{ "subsong_timeout", &subsong_timeout, cfg_long },
+	{ "verbosity", &verbosity, cfg_long },
 	/* playmode not implemented yet */
 	{ NULL, NULL, NULL }
 };
@@ -340,7 +341,7 @@ static regparm void usage(long exitcode)
 {
 	FILE *out = exitcode ? stderr : stdout;
 	fprintf(out,
-	        _("Usage: %s [option(s)] <gbs-file> [start_at_subsong [stop_at_subsong] ]\n"
+		_("Usage: %s [option(s)] <gbs-file> [start_at_subsong [stop_at_subsong] ]\n"
 		  "\n"
 		  "Available options are:\n"
 		  "  -E        endian, b == big, l == little, n == native (%s)\n"
@@ -360,15 +361,15 @@ static regparm void usage(long exitcode)
 		  "  -z        play subsongs in shuffle mode\n"
 		  "  -Z        play subsongs in random mode (repetitions possible)\n"
 		  "  -1 to -4  mute a channel on startup\n"),
-	        myname,
-	        endian_str(endian),
+		myname,
+		endian_str(endian),
 		fadeout,
 		subsong_gap,
 		sound_name,
-	        rate,
+		rate,
 		refresh_delay,
 		subsong_timeout,
-	        silence_timeout);
+		silence_timeout);
 	exit(exitcode);
 }
 
@@ -382,7 +383,7 @@ static regparm void parseopts(int *argc, char ***argv)
 {
 	long res;
 	myname = *argv[0];
-	while ((res = getopt(*argc, *argv, "1234E:f:g:hlo:qr:R:t:T:vVzZ")) != -1) {
+	while ((res = getopt(*argc, *argv, "1234c:E:f:g:hlo:qr:R:t:T:vVzZ")) != -1) {
 		switch (res) {
 		default:
 			usage(1);
@@ -393,6 +394,9 @@ static regparm void parseopts(int *argc, char ***argv)
 		case '4':
 			gbhw_ch[res-'1'].mute ^= 1;
 			break;
+		case 'c':
+			cfg_parse(optarg, options);
+			break;
 		case 'E':
 			if (strcasecmp(optarg, "b") == 0) {
 				endian = PLUGOUT_ENDIAN_BIG;
@@ -482,6 +486,7 @@ static regparm void handleuserinput(struct gbs *gbs)
 		case ' ':
 			pause_mode = !pause_mode;
 			gbhw_pause(pause_mode);
+			if (sound_pause) sound_pause(pause_mode);
 			break;
 		case '1':
 		case '2':
@@ -549,7 +554,7 @@ static regparm void printstatus(struct gbs *gbs)
 		songtitle=_("Untitled");
 	}
 	printf("\r\033[A\033[A"
-	       "Song %3ld/%3ld (%s)\033[K\n"
+	       "Song %3d/%3d (%s)\033[K\n"
 	       "%02ld:%02ld/%02ld:%02ld",
 	       gbs->subsong+1, gbs->songs, songtitle,
 	       timem, times, lenm, lens);
@@ -630,6 +635,7 @@ static regparm void select_plugin(void)
 	sound_io = plugout->io;
 	sound_write = plugout->write;
 	sound_close = plugout->close;
+	sound_pause = plugout->pause;
 	sound_description = plugout->description;
 
 	if (plugout->flags & PLUGOUT_USES_STDOUT) {
