@@ -1,5 +1,4 @@
-/* $Id: plugout_stdout.c,v 1.8 2005/08/06 21:33:16 ranmachan Exp $
- *
+/*
  * gbsplay is a Gameboy sound player
  *
  * 2004 (C) by Christian Garbs <mitch@cgarbs.de>
@@ -16,7 +15,8 @@
 
 int fd;
 
-static long regparm stdout_open(enum plugout_endian endian, long rate)
+static long regparm stdout_open(/*@unused@*/ enum plugout_endian endian,
+                                /*@unused@*/ long rate)
 {
 	/*
 	 * clone and close STDOUT_FILENO
@@ -24,7 +24,7 @@ static long regparm stdout_open(enum plugout_endian endian, long rate)
 	 */
 	fd = dup(STDOUT_FILENO);
 	if (fd == -1) return -1;
-	close(STDOUT_FILENO);
+	(void)close(STDOUT_FILENO);
 
 	return 0;
 }
@@ -36,7 +36,7 @@ static ssize_t regparm stdout_write(const void *buf, size_t count)
 
 static void regparm stdout_close()
 {
-	close(fd);
+	(void)close(fd);
 }
 
 const struct output_plugin plugout_stdout = {
